package installment_payment

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	enum2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/installment_payment/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/installment_payment/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/implement"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	"errors"
	"fmt"
	"time"
	"xorm.io/xorm"
)

const LockKey = "installment_payment_auto_repaid_lock_key"

// InstallmentPaymentAutoRepaid 分期付自动还款
func InstallmentPaymentAutoRepaid(engine *xorm.Engine, masterId string, ch *rabbit.Channel) (err error) {
	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(LockKey)
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次结算未执行完")
		return errors.New("上一次结算未执行完")
	}
	cache.SetEx(LockKey, "running", 3600*1) //1小时

	//1、查找出"可用余额"大于"每期还款金额"的用户数据
	var sqlStr = "SELECT ipl.*, up.fin_valid, up.uid FROM `installment_payment_list` ipl " +
		"JOIN `user_profile` up ON up.uid = ipl.uid " +
		"WHERE ipl.is_repaid_off =0 AND up.fin_valid > ipl.repayment_amount_per_installment;"
	nativeStringResultMap, _ := db.QueryNativeString(engine, sqlStr)

	//2、整理用户余额数据
	var userFinValid = map[string]float64{}
	for _, resultMap := range nativeStringResultMap {
		userFinValid[resultMap["uid"]] = zhios_order_relate_utils.StrToFloat64(resultMap["fin_valid"])
	}

	//2、将符合条件的数据推入mq队列
	for _, resultMap := range nativeStringResultMap {
		if userFinValid[resultMap["uid"]] >= zhios_order_relate_utils.StrToFloat64(resultMap["repayment_amount_per_installment"]) {
			//TODO::推入rabbitmq 异步处理
			ch.Publish(md.InstallmentPaymentExchange, md.InstallmentPaymentStructForAutoRepaid{
				MasterId:                      masterId,
				Id:                            zhios_order_relate_utils.StrToInt(resultMap["id"]),
				Uid:                           zhios_order_relate_utils.StrToInt(resultMap["uid"]),
				GoodsId:                       zhios_order_relate_utils.StrToInt(resultMap["goods_id"]),
				OrdId:                         zhios_order_relate_utils.StrToInt64(resultMap["ord_id"]),
				TotalAmount:                   resultMap["total_amount"],
				InstallmentNums:               zhios_order_relate_utils.StrToInt(resultMap["installment_nums"]),
				AlreadyRepaidInstallmentNums:  zhios_order_relate_utils.StrToInt(resultMap["already_repaid_installment_nums"]),
				RepaymentAmountPerInstallment: resultMap["repayment_amount_per_installment"],
				IsRepaidOff:                   zhios_order_relate_utils.StrToInt(resultMap["is_repaid_off"]),
			}, md.InstallmentPaymentRoutKeyForAutoRepaid)
			userFinValid[resultMap["uid"]] -= zhios_order_relate_utils.StrToFloat64(resultMap["repayment_amount_per_installment"])
		}
	}

	cache.Del(LockKey) //删除悲观锁
	return
}

// DealInstallmentPaymentAutoRepaid 处理分期付自动还款
func DealInstallmentPaymentAutoRepaid(engine *xorm.Engine, ch *rabbit.Channel, args md.InstallmentPaymentStructForAutoRepaid, masterId string) (err error) {
	now := time.Now()
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	//1、查询出对应`installment_payment_list` 数据
	installmentPaymentListDb := implement.NewInstallmentPaymentListDb(engine)
	installmentPaymentList, err := installmentPaymentListDb.GetInstallmentPaymentListById(args.Id)
	if err != nil {
		session.Rollback()
		return err
	}
	installmentPaymentList.AlreadyRepaidInstallmentNums++
	if installmentPaymentList.AlreadyRepaidInstallmentNums == installmentPaymentList.InstallmentNums {
		installmentPaymentList.IsRepaidOff = int(enum2.InstallmentPaymentListRepaidOn) //TODO::已还完
	}

	//2、处理用户余额
	orderType := enum.FinUserFlowOrderActionString(md2.InstallmentPaymentAutoRepaidForFinUserFlow)
	var dealUserAmount = md2.DealUserAmount{
		Kind:      "sub",
		Mid:       masterId,
		Title:     md2.InstallmentPaymentAutoRepaidTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md2.InstallmentPaymentAutoRepaidForFinUserFlow,
		OrdId:     zhios_order_relate_utils.Int64ToStr(args.OrdId),
		Uid:       args.Uid,
		Amount:    zhios_order_relate_utils.StrToFloat64(installmentPaymentList.RepaymentAmountPerInstallment),
	}
	err = svc.DealUserAmount(session, dealUserAmount)
	if err != nil {
		session.Rollback()
		return err
	}

	//3、新增`installment_payment_repaid_flow`数据
	oid := zhios_order_relate_utils.OrderUUID(installmentPaymentList.Uid)
	installmentPaymentRepaidFlowDb := implement.NewInstallmentPaymentRepaidFlowDb(engine)
	_, err = installmentPaymentRepaidFlowDb.InsertInstallmentPaymentRepaidFlowBySession(session, &models.InstallmentPaymentRepaidFlow{
		RecordId:             installmentPaymentList.Id,
		Uid:                  installmentPaymentList.Uid,
		Amount:               installmentPaymentList.RepaymentAmountPerInstallment,
		WhichRepaymentPeriod: installmentPaymentList.AlreadyRepaidInstallmentNums,
		CreateAt:             now.Format("2006-01-02 15:04:05"),
		UpdateAt:             now.Format("2006-01-02 15:04:05"),
		OrdId:                zhios_order_relate_utils.StrToInt64(oid),
	})
	if err != nil {
		session.Rollback()
		return err
	}

	//4、修改`installment_payment_list` 记录
	updateAffected, err := installmentPaymentListDb.UpdateInstallmentPaymentListBySess(session, installmentPaymentList, "already_repaid_installment_nums", "is_repaid_off")
	if err != nil {
		session.Rollback()
		return err
	}
	if updateAffected <= 0 {
		session.Rollback()
		return errors.New("更新 `installment_payment_list` 记录失败")
	}
	err = session.Commit()
	if err != nil {
		session.Rollback()
		return err
	}
	rewardData := md.ZhiosTaskReward{
		Uid:      zhios_order_relate_utils.IntToStr(installmentPaymentList.Uid),
		Mid:      masterId,
		Reward:   zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(installmentPaymentList.Commission)),
		Money:    zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(installmentPaymentList.RepaymentAmountPerInstallment)),
		Title:    "分期付自动还款",
		Oid:      oid,
		Provider: "installment_payment",
		PvdOid:   zhios_order_relate_utils.Int64ToStr(args.OrdId),
	}
	err1 := ch.PublishV2(md.RelateReward, zhios_order_relate_utils.SerializeStr(rewardData), md.RelateRewardData)
	if err1 != nil {
		ch.PublishV2(md.RelateReward, zhios_order_relate_utils.SerializeStr(rewardData), md.RelateRewardData)
	}
	return
}
