package service_award_dividend

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/service_award_dividend/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/implement"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	"encoding/json"
	"errors"
	"time"
	"xorm.io/xorm"
)

func Init(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md2.RedisDataBase)
	return
}

func JoinServiceAwardDividendRelation(eg *xorm.Engine, req md.JoinServiceAwardDividendRelationReq) (err error) {
	user, err := db.DbsUserFindByUid(eg, req.Uid)
	if err != nil {
		return err
	}
	if user == nil {
		return errors.New("未查询到相应会员记录")
	}

	serviceAwardDividendBasicDb := implement.NewServiceAwardDividendBasicDb(eg)
	base, _ := serviceAwardDividendBasicDb.GetServiceAwardDividendBasic()
	if base.IsOpen != 1 {
		err = errors.New("服务奖分红未开启！")
		return
	}
	var vipAllocationSetStruct []md.VipAllocationSetStruct //会员分配设置
	err = json.Unmarshal([]byte(base.VipAllocationSet), &vipAllocationSetStruct)
	if err != nil {
		return err
	}
	if req.CreateAt == "" {
		req.CreateAt = time.Now().Format("2006-01-02 15:04:05")
	}

	//1、判断是否"推荐人优先"
	if req.ParentUid != 0 && base.AllocationPriority == 1 {
		//2、判断当前用户和父级用户谁的会员等级更高
		parentUser, err := db.DbsUserFindByUid(eg, req.ParentUid)
		if err != nil {
			return err
		}
		var levelMap = map[int]model.UserLevel{}
		lvList, err := db.UserLevelInIDescByWeightLowV1(eg)
		for _, v := range lvList {
			levelMap[v.Id] = *v
		}

		if levelMap[user.Level].LevelWeight >= levelMap[parentUser.Level].LevelWeight {
			for _, vv := range vipAllocationSetStruct {
				if zhios_order_relate_utils.StrToInt(vv.FromVipId) == user.Level {
					err = AutoAddVip(eg, user, req.ParentUid, vv, req.RegionId, req.CreateAt, req.AllocationNums)
					if err != nil {
						return err
					}
				}
			}
		} else {
			err = AddVipToParent(eg, user, req.ParentUid, req.RegionId, req.CreateAt, req.AllocationNums)
			if err != nil {
				return err
			}
		}
	} else {
		for _, vv := range vipAllocationSetStruct {
			if zhios_order_relate_utils.StrToInt(vv.FromVipId) == user.Level {
				err = AutoAddVip(eg, user, req.ParentUid, vv, req.RegionId, req.CreateAt, req.AllocationNums)
				if err != nil {
					return err
				}
			}
		}
	}

	return
}

// AutoAddVip 自动添加会员
func AutoAddVip(engine *xorm.Engine, user *model.User, parentUid int, vipAllocationSetStruct md.VipAllocationSetStruct, regionId int, createAt string, allocationNums int) (err error) {
	//1、查询出当前应该分配给哪个上级会员
	var serviceAwardDividendRelation models.ServiceAwardDividendRelation
	var hasServiceAwardDividendRelation bool
	if regionId != 0 {
		hasServiceAwardDividendRelation, err = engine.Where("region_id =?", regionId).In("vip_level", vipAllocationSetStruct.ToVipId).Asc("allocation_nums").Get(&serviceAwardDividendRelation)
		if err != nil {
			return err
		}
	}
	if !hasServiceAwardDividendRelation {
		hasServiceAwardDividendRelation, err = engine.In("vip_level", vipAllocationSetStruct.ToVipId).Asc("allocation_nums").Get(&serviceAwardDividendRelation)
		if err != nil {
			return err
		}
	}

	now := time.Now()
	sess := engine.NewSession()
	defer func() {
		sess.Close()
		if err := recover(); err != nil {

			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	sess.Begin()

	if hasServiceAwardDividendRelation == false {
		//return errors.New("未查询到合适的绑定上级会员用户！")
		//TODO::未查询到合适的绑定上级会员用户时(新增 service_award_dividend_relation 记录)
		serviceAwardDividendRelationDb := implement.NewServiceAwardDividendRelationDb(engine)
		_, err = serviceAwardDividendRelationDb.InsertServiceAwardDividendRelationBySess(sess, &models.ServiceAwardDividendRelation{
			Uid:            user.Uid,
			ParentUid:      parentUid,
			BindUid:        0,
			VipLevel:       user.Level,
			AllocationNums: allocationNums,
			RegionId:       regionId,
			CreateAt:       createAt,
			UpdateAt:       now.Format("2006-01-02 15:04:05"),
		})
		if err != nil {
			sess.Rollback()
			return err
		}
		return
	} else {
		//2、新增 service_award_dividend_relation 记录
		serviceAwardDividendRelationDb := implement.NewServiceAwardDividendRelationDb(engine)
		_, err = serviceAwardDividendRelationDb.InsertServiceAwardDividendRelationBySess(sess, &models.ServiceAwardDividendRelation{
			Uid:            user.Uid,
			ParentUid:      parentUid,
			BindUid:        serviceAwardDividendRelation.Uid,
			VipLevel:       user.Level,
			AllocationNums: allocationNums,
			RegionId:       regionId,
			CreateAt:       createAt,
			UpdateAt:       now.Format("2006-01-02 15:04:05"),
		})
		if err != nil {
			sess.Rollback()
			return err
		}

		//3、修改 绑定上级的 service_award_dividend_relation 记录
		serviceAwardDividendRelation.AllocationNums++
		updateRelationAfftecd, err := serviceAwardDividendRelationDb.SaveServiceAwardDividendRelation(sess, serviceAwardDividendRelation.Id, &serviceAwardDividendRelation, "allocation_nums")
		if err != nil {
			return err
		}
		if updateRelationAfftecd <= 0 {
			return errors.New("更新 绑定上级的 service_award_dividend_relation 记录 失败！")
		}
	}

	return sess.Commit()
}

// AddVipToParent 添加会员至父级
func AddVipToParent(engine *xorm.Engine, user *model.User, parentUid int, regionId int, createAt string, allocationNums int) (err error) {
	//1、查询出当前应该分配给哪个上级会员
	var serviceAwardDividendRelation models.ServiceAwardDividendRelation
	has, err := engine.Where("uid =?", parentUid).Get(&serviceAwardDividendRelation)
	if err != nil {
		return err
	}
	if has == false {
		return errors.New("未查询到对应绑定上级会员用户！")
	}

	now := time.Now()
	sess := engine.NewSession()
	defer func() {
		sess.Close()
		if err := recover(); err != nil {

			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	sess.Begin()

	//2、新增 service_award_dividend_relation 记录
	serviceAwardDividendRelationDb := implement.NewServiceAwardDividendRelationDb(engine)
	_, err = serviceAwardDividendRelationDb.InsertServiceAwardDividendRelationBySess(sess, &models.ServiceAwardDividendRelation{
		Uid:            user.Uid,
		ParentUid:      parentUid,
		BindUid:        serviceAwardDividendRelation.Uid,
		VipLevel:       user.Level,
		AllocationNums: allocationNums,
		RegionId:       regionId,
		CreateAt:       createAt,
		UpdateAt:       now.Format("2006-01-02 15:04:05"),
	})
	if err != nil {
		sess.Rollback()
		return err
	}

	//3、修改 绑定上级的 service_award_dividend_relation 记录
	serviceAwardDividendRelation.AllocationNums++
	updateRelationAfftecd, err := serviceAwardDividendRelationDb.SaveServiceAwardDividendRelation(sess, serviceAwardDividendRelation.Id, &serviceAwardDividendRelation, "allocation_nums")
	if err != nil {
		return err
	}
	if updateRelationAfftecd <= 0 {
		return errors.New("更新 绑定上级的 service_award_dividend_relation 记录 失败！")
	}
	return sess.Commit()
}
